/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTActorClass_h__
#define __RTActorClass_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTVersionId_h__
#include <RTVersionId.h>
#endif

class  RTActor;
class  RTActorRef;
struct RTComponentDescriptor;
class  RTController;
struct RTRelayDescriptor;

// This structure describes the external characteristics of an actor class.
//
// Field        Meaning
// -----        -------
// super        The superclass descriptor.
// name         The name of the actor class.
// version      A version identifier, updated with the actor class.
// numRelays    The number of interfaces (relay ports) provided by
//              the actor class.
// relays       An array of descriptors for those interfaces.
// create       Points to a constructor function.

typedef RTActor * ( * RTActorCreator )( RTController *, RTActorRef * );

struct RTActorClass
{
    const RTActorClass      * super;
    const char              * name;
    RTVersionId               version;

    int                       numRelays;
    const RTRelayDescriptor * relays;

    RTActorCreator            create;

    int isKindOf( const RTActorClass * superclass ) const;
    int substitutableFor( const RTComponentDescriptor * ) const;
};

extern const RTActorClass EmptyActorClass;

#endif // __RTActorClass_h__
