/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTActor_class_h__
#define __RTActor_class_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

struct RTActorClass;
struct RTComponentDescriptor;
struct RTFieldDescriptor;
struct RTLocalBindingDescriptor;
struct RTPortDescriptor;

// This structure describes the internal characteristics of an actor class.
//
// Field            Meaning
// -----            -------
// super            The base type of this type.
// names            An array of names of the states and the choice points
//                  (grouped in that order).
// numStates        The number of states including "top" which is state one.
// parentState      An array which when indexed by state id yields the state
//                  id of the parent state.
// type             The external descriptor.
// numComponents    The number of (actor) components.
// components       An array of component descriptors.
// numPorts         The total number of ports.
// ports            An array of port descriptors.
// numBindings      The total number of bindings involving only end ports
//                  of the actor class.
// bindings         An array of local binding descriptors.
// numAttributes    The total number of attributes.
// attributes       An array of descriptors of the attributes.

struct RTActor_class
{
    const RTActor_class               * super;

    const char * const                * names;

    int                                 numStates;
    const RTStateId                   * parentState;

    const RTActorClass                * type;

    int                                 numComponents;
    const RTComponentDescriptor       * components;

    int                                 numPorts;
    const RTPortDescriptor            * ports; // ordered by group, id;
					       // with the relay group first

    int                                 numBindings;
    const RTLocalBindingDescriptor    * bindings;

    int                                 numAttributes;
    const RTFieldDescriptor           * attributes;

    const RTFieldDescriptor           * findAttribute( const char * ) const;
};

#endif // __RTActor_class_h__
