#! perl -w

# Licensed Materials - Property of IBM                       
# Component IBM Rational ROSE_RT BUILD : TARGET_RTS                 
# Copyright IBM Corporation 1999, 2008. All Rights Reserved.      
# U.S. Government Users Restricted Rights -  Use, duplication or    
# disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.



# This script is necessary because Windows-NT cannot take
# more than certain number of characters in a command line.

sub fatal
{
    print STDERR "$_[0]\n";
    exit 1;
}

$#ARGV >= 1 || &fatal( "Usage: ${0} linkppc options ... " );

@command  = ( shift );
@usrlib_dirs = ();
if( $lib = $ENV{'LIB'} )
{
    @lib_dirs = split( ';', $lib );
}
else
{
    @lib_dirs = ();
}
$status   = 0; # assume all is well

sub find_lib
{
    my( $lib ) = @_;
    my( $file );

    foreach $dir ( @usrlib_dirs, @lib_dirs )
    {
	$file = "$dir/$lib.lib";
	if( -r "$file" )
	{
	    return $file;
	}
    }

    return undef();
}

$out = "ld.cmd";
if( ! open( OUT, ">$out" ) )
{
    print STDERR "Can't create $out\n";
    exit 1;
}

while( $#ARGV >= 0 )
{
    $arg = shift;
    if( $arg =~ m{^-l(.*)$} )
    {
	if( $lib = &find_lib( $1 ) )
	{
	    print OUT "load $lib\n";
	}
	else
	{
	    print STDERR "Can't find library for $arg\n";
	    $status = 1;
	}
    }
    elsif( $arg =~ m{^-L(.*)$} )
    {
	push @usrlib_dirs, $1;
    }
    elsif( $arg =~ m{^%(.*)$} )
    {
	$in = $1;
	if( ! open( IN, "<$in" ) )
	{
	    print STDERR "Can't read '$in': $!\n";
	    $status = 1;
	}
	else
	{
	    while( <IN> )
	    {
		chomp;
		print OUT "load $_.o\n";
	    }
	    close( IN );
	}
    }
    elsif( $arg =~ m{^-[cfou]} )
    {
	if( $#ARGV < 0 )
	{
	    print STDERR "Value required for '$arg'\n";
	    $status = 1;
	}
	else
	{
	    push @command, $arg, shift;
	}
    }
    elsif( $arg =~ m{^-} )
    {
	push @command, $arg;
    }
    else
    {
	print OUT "load $arg\n";
    }
}

close( OUT );

push @command, '-c', $out;

if( $status == 0 )
{
    $status = system @command;
}

unlink( $cmd_file );

if( $status < 256 )
{
    exit $status;
}
else
{
    exit $status / 256;
}
