/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTController.h>
#include <RTDebugger.h>
#include <RTEventInfo.h>
#include <RTMessage.h>

void RTActorProbe::evtChain( int chainPos )
{
    if( _chainOrigin <= 0 )
	return;

    int currState = _actor->getCurrentState();

    if( chainPos == 1 ) // first chain, before state exit code
    {
	RTEventInfo * evtFSM = new RTEventInfo( RTEventInfo::EventFSM,
						_id,
						_actorIndex );

	_chainParent = currState;
	_debugger->getTimestamp( evtFSM->timestamp );
	evtFSM->message       = new RTMessage( *_chainMsg );
	evtFSM->chainOrigin   = _chainOrigin;
	evtFSM->chainStrTrans = _chainTrans;
	evtFSM->chainParent   = _chainParent;

	context()->breakPoint( _actor, evtFSM );
    }
    else if( chainPos == 2 ) // second chain, before transition code
	_chainState = currState;

    RTEventInfo * eventChain = new RTEventInfo( RTEventInfo::EventChain,
						_id,
						_actorIndex );

    _debugger->getTimestamp( eventChain->timestamp );
    eventChain->message       = new RTMessage( *_chainMsg );
    eventChain->chainOrigin   = _chainOrigin;
    eventChain->chainStrTrans = _chainTrans;
    eventChain->chainPosition = chainPos;
    eventChain->chainParent   = _chainParent;

    context()->breakPoint( _actor, eventChain );

    if( chainPos == 0 ) // final chain, after state entry code
    {
	// LateFSM: currently used only by MSC daemons. Do _not_ update
	// _chainParent, its value (from when chainPos == 1) is OK.
	RTEventInfo * evtLateFSM = new RTEventInfo( RTEventInfo::EventLateFSM,
						    _id,
						    _actorIndex );

	_debugger->getTimestamp( evtLateFSM->timestamp );
	evtLateFSM->message       = new RTMessage( *_chainMsg );
	evtLateFSM->chainOrigin   = _chainOrigin;
	evtLateFSM->chainStrTrans = _chainTrans;
	evtLateFSM->chainParent   = _chainParent;

	context()->breakPoint( _actor, evtLateFSM );

	// We don't want to catch internal non-ex(c)iting self-transitions.
	// If we pass through OR end up in a different state,
	if( currState != _chainState || currState != _chainParent  )
	{
	    // If there is a State Probe on the current state, send evtState.
	    RTEventInfo * evtState = new RTEventInfo( RTEventInfo::EventState,
						      _id,
						      _actorIndex );

	    _debugger->getTimestamp( evtState->timestamp );
	    evtState->message     = new RTMessage( *_chainMsg );
	    evtState->chainParent = _chainParent;
	    evtState->stateTarget = currState;

	    context()->breakPoint( _actor, evtState );
	}

	// If we pass through a different state but end up in the same
	if( currState != _chainState && currState == _chainParent )
	{
	    // If there is a Chain Probe (= Behavior Monitor window) on
	    // the _chainState, send evtHistory to update the Monitor window.
	    RTEventInfo * evtHist = new RTEventInfo( RTEventInfo::EventHistory,
						     _id,
						     _actorIndex );

	    evtHist->stateTarget = _chainState;
	    evtHist->histHist    = _actor->getHistory( _chainState );
	    evtHist->histCurr    = currState;

	    context()->breakPoint( _actor, evtHist );
	}

	_chainOrigin = 0;
	_chainMsg    = (const RTMessage *)0;
    }
}
