/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActorRef.h>
#include <RTActorSlot.h>
#include <RTComponentDescriptor.h>
#include <RTDiag.h>
#include <RTImportList.h>

int RTActorRef::reserve( int & index )
{
    int replication = _info->replication;

    if( index >= replication )
	return 0; // index out of range

    int status = 1;

#if USE_THREADS
    lock();
#endif

    switch( _info->kind )
    {
    case RTComponentDescriptor::Imported:
    {
	RTImportList * entry;

	if( replication == 1 )
	{
	    // We don't have a free list, try to allocate the only entry.
	    entry = (RTImportList *)_contents;
	    if( entry->used.reference == this )
	    {
		status = 0; // already allocated
		break;
	    }
	    index = 0;
	}
	else
	{
	    RTImportList::Unused * head  = (RTImportList::Unused *)_contents;
	    RTImportList         * first = (RTImportList *)(void *)&head[ 1 ];

	    if( index < 0 )
	    {
		if( head == (RTImportList::Unused *)head->next )
		{
		    status = 0; // full
		    break;
		}

		entry = head->next;
		index = entry - first; //lint !e946 !e947
	    }
	    else
	    {
		entry = &first[ index ];

		if( entry->used.reference == this )
		{
		    status = 0; // already allocated
		    break;
		}
	    }

	    entry->unused.prev->unused.next = entry->unused.next;
	    entry->unused.next->unused.prev = entry->unused.prev;
	}

	entry->used.reference = this;
	break;
    }

    case RTComponentDescriptor::Optional:
	if( replication == 1 )
	{
	    if( _contents != (void *)0 )
		status = 0; // in use
	    // We don't have a free list, allocate the only entry.
	    index = 0;
	}
	else if( RTSmallOptional::supports( replication ) != 0 )
	{
	    RTSmallOptional * head  = (RTSmallOptional *)_contents;
	    RTSmallOptional * entry;

	    if( index < 0 )
	    {
		int iReserved = head->next();

		if( iReserved == 0 )
		{
		    status = 0; // full
		    break;
		}

		index = iReserved - 1;
		entry = &head[ iReserved ];
	    }
	    else
	    {
		entry = &head[ index + 1 ];
		if( entry->inuse() != 0 )
		{
		    status = 0; // index already allocated
		    break;
		}
	    }

	    int               iPrev = entry->prev();
	    int               iNext = entry->next();
	    RTSmallOptional * prev  = &head[ iPrev ];
	    RTSmallOptional * next  = &head[ iNext ];

	    prev->next( iNext );
	    next->prev( iPrev );

	    entry->_actor = (RTActor *)0;
	}
	else
	{
	    //lint -save -e661
	    RTLargeOptional * head  = (RTLargeOptional *)_contents;
	    RTLargeOptional * entry;

	    if( index < 0 )
	    {
		entry = head->next;

		if( entry == head )
		{
		    status = 0; // full
		    break;
		}

		index = ( entry - head ) - 1; //lint !e946 !e947
	    }
	    else
	    {
		entry = &head[ index + 1 ];

		if( entry->next == (RTLargeOptional *)0 )
		{
		    status = 0; // index already allocated
		    break;
		}
	    }

	    entry->prev->next = entry->next;
	    entry->next->prev = entry->prev;

	    entry->next  = (RTLargeOptional *)0;
	    entry->actor = (RTActor         *)0;
	    //lint -restore
	}
	break;

    case RTComponentDescriptor::Fixed:
	if( index < 0 )
	{
	    // We are always specific with fixed components,
	    // this must be the application doing something illegal.
	    status = 0;
	}
	else if( replication == 1 )
	{
	    if( _contents != (void *)0 )
		status = 0; // in use
	}
	else if( ((RTActor **)_contents)[ index ] != (RTActor *)0 )
	{
	    status = 0; // in use
	}
	break;

    default:
	status = 0;
	break;
    }

#if USE_THREADS
    unlock();
#endif

    return status;
}
