/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTAsciiDecoding.h>
#include <RTAsciiEncoding.h>
#include <RTMemoryUtil.h>
#include <RTObject_class.h>
#include <RTUnknownObject.h>

enum Action
{
    SaveAndContinue
  , SaveAndQuit
  , JustQuit
};

enum QState
{
    QOutside
  , QSingleInside, QSingleEscape
  , QDoubleInside, QDoubleEscape
};

RTUnknownObject * RTAsciiDecoding::getUnknown( const char * name )
{
    int      name_size   = RTMemoryUtil::strlen( name );
    int      buffer_size = name_size + 4096 + 3;
    char   * buffer      = new char [ (unsigned)buffer_size ];
    int      size        = name_size;
    Action   action      = SaveAndContinue;
    QState   quote_state = QOutside;
    int      nest_level  = 0;
    char     byte;

    if( buffer == (char *)0 )
	return (RTUnknownObject *)0;

    RTMemoryUtil::memcpy( buffer, name, size );

    while( getByte( byte ) != 0 )
    {
	switch( quote_state )
	{
	case QOutside:
	    if     ( byte ==  '"' ) quote_state = QDoubleInside;
	    else if( byte == '\'' ) quote_state = QSingleInside;
	    else if( byte ==  '{' )
		++nest_level;
	    else if( byte == '}' )
	    {
		if( nest_level == 0 )
		    action = JustQuit;
		else if( --nest_level == 0 )
		    action = SaveAndQuit;
	    }
	    else if( byte == ',' )
	    {
		if( nest_level == 0 )
		    action = JustQuit;
	    }
	    break;

	case QSingleInside:
	    if     ( byte == '\'' ) quote_state = QOutside;
	    else if( byte == '\\' ) quote_state = QSingleEscape;
	    break;

	case QDoubleInside:
	    if     ( byte ==  '"' ) quote_state = QOutside;
	    else if( byte == '\\' ) quote_state = QDoubleEscape;
	    break;

	case QSingleEscape: quote_state = QSingleInside; break;
	case QDoubleEscape: quote_state = QDoubleInside; break;

	default:
	    break;
	}

	if( action == JustQuit )
	{
	    ungetByte( byte );
	    break;
	}

	if( size >= buffer_size )
	{
	    buffer_size += 4096;

	    char * new_buffer = new char [ (unsigned)buffer_size ];

	    if( new_buffer == (char *)0 )
	    {
		delete [] buffer;
		return (RTUnknownObject *)0;
	    }

	    RTMemoryUtil::memcpy( new_buffer, buffer, size );
	    delete [] buffer;
	    buffer = new_buffer;
	}

	buffer[ size++ ] = byte;

	if( action == SaveAndQuit )
	    break;
    }

    RTUnknownObject * unknown = new RTUnknownObject;

    unknown->signature = RTMemoryUtil::strdup( "RTAsciiDecoding" );
    unknown->length    = size;
    unknown->code      = (char *)RTMemoryUtil::memdup( buffer, size );

    delete [] buffer;

    return unknown;
}
