/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActorClass.h>
#include <RTBindingDescriptor.h>
#include <RTComponentDescriptor.h>
#include <RTController.h>
#include <RTInterfaceDescriptor.h>
#include <RTProtocolDescriptor.h>
#include <RTRelayDescriptor.h>

RTController::Error
RTController::canPlugIn( const RTActorClass          * candidate,
			 const RTComponentDescriptor * role )
{
    if( role->substitutable != 0 )
    {
	if( candidate->isKindOf( role->defaultClass ) == 0 )
	    return RTController::badClass;
    }
    else
    {
	if( role->defaultClass != candidate )
	    return RTController::badClass;
    }

    // check interfaces and sufficient replication available
    {
	const RTRelayDescriptor     * relay    = candidate->relays;
	const RTInterfaceDescriptor * required = role->interfaces;

	int relayIndex = role->numInterfaces;

	if( relayIndex < 0 || relayIndex > candidate->numRelays )
	    return RTController::badInterface;

	for( ; --relayIndex >= 0; ++relay, ++required )
	    if( relay->replication < required->replication )
		return RTController::smallInterface;
    }

    RTController::Error result = RTController::ok;

    // check compatibility of protocols
    {
	const RTRelayDescriptor   * relay   = candidate->relays;
	const RTBindingDescriptor * binding = role->bindings;

	for( int i = role->numBindings; --i >= 0; ++binding )
	{
	    int relayIndex = binding->relay;

	    if( relayIndex < 0 || relayIndex >= candidate->numRelays )
		return RTController::badInterface;

	    switch( binding->protocol
			   ->compare( relay[ relayIndex ].protocol ) )
	    {
	    default:
	    case RTProtocolDescriptor::badSignal:
		return RTController::protocolSignal;

	    case RTProtocolDescriptor::badType:
		result = RTController::protocolType;
		break;

	    case RTProtocolDescriptor::ok:
		break;
	    }
	}
    }

    return result;
}
