/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTCmdLineObserver.h>
#include <RTDebugger.h>
#include <RTMemoryUtil.h>
#include <RTObserver.h>
#include <RTTcpSocket.h>
#include <RTToolsetObserver.h>
#include <RTWebObserver.h>

RTObserver * RTDebugger::findObserver( RTTcpSocket * socket, int obsId )
{
    int  numRead;
    char firstCh;

    switch( socket->state() )
    {
    case RTTcpSocket::Established:
	if( socket->hasData() == 0 )
	    break;

	numRead = socket->read( &firstCh, 1 );

	if( numRead <= 0 ) // instantiate base class, will be deleted soon
	    return new RTObserver( this, socket, obsId );

	// try to figure out what kind of observer we need
	if( firstCh == '\r' || firstCh == '\n' ) // CR or LF: telnet
	    return new RTCmdLineObserver( this, socket, obsId, &firstCh, 1 );

	if( '0' <= firstCh && firstCh <= '9' ) // 0..9: toolset
	    return new RTToolsetObserver( this, socket, obsId, &firstCh, 1 );

	if( firstCh == 'G' || firstCh == 'P' ) // GET or POST: web
	    return new RTWebObserver( this, socket, obsId, &firstCh, 1 );

	socket->close();
	// fall through

    case RTTcpSocket::Closed:
	// instantiate base class, will be deleted soon
	return new RTObserver( this, socket, obsId );

#ifdef _lint
    case RTTcpSocket::Listen:
#endif
    case RTTcpSocket::SynSent:
    default:
	break;
    }

    return (RTObserver *)0;
}
