/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTController.h>
#include <RTDiag.h>
#include <RTLayerConnector.h>
#include <RTMessage.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>
#include <RTQFilter.h>

class RTProtocol_resize_filter : public RTQFilter
{
public:
    RTProtocol_resize_filter( RTController * _rts,
			      RTProtocol   * _port,
			      int            _size )
	    : RTQFilter()
	    , rts ( _rts  )
	    , port( _port )
	    , size( _size )
	{
	}
    virtual ~RTProtocol_resize_filter( void );

    virtual int  select( RTMessage & );
    virtual void accept( RTMessage & );

private:
    RTController * rts;
    RTProtocol   * port;
    int            size;

    // unavailable methods
    RTProtocol_resize_filter( const RTProtocol_resize_filter & );
    RTProtocol_resize_filter & operator=( const RTProtocol_resize_filter & );
}; //lint !e1712

int RTProtocol_resize_filter::select( RTMessage & msg )
{
    return msg.sap()       == port
	&& msg.sapIndex0() >= size;
}

void RTProtocol_resize_filter::accept( RTMessage & msg )
{
    rts->freeMsg( &msg );
}

RTProtocol_resize_filter::~RTProtocol_resize_filter( void )
{
    rts  = (RTController *)0;
    port = (RTProtocol   *)0;
}

//lint -save -e1738
RTProtocol_resize_filter::
RTProtocol_resize_filter( const RTProtocol_resize_filter & )
    : RTQFilter()
    , rts ( (RTController *)0 )
    , port( (RTProtocol   *)0 )
    , size(                 0 )
{
    RTDiag::panic( "Unavailable method called:\n"
"    'RTProtocol_resize_filter( const RTProtocol_resize_filter & )'" );
}
//lint -restore

RTProtocol_resize_filter &
RTProtocol_resize_filter::operator=( const RTProtocol_resize_filter & )
{
    RTDiag::panic( "Unavailable method called:\n"
"    'RTProtocol_resize_filter::operator=( const RTProtocol_resize_filter & )'" );

    //lint -unreachable
    return *this;
} //lint !e1529

int RTProtocol::resize( int new_size )
{
    RTActor                 * owner      = getOwner();
    RTController            * controller = owner->context();
    int                       old_size   = _info->replication;
    RTController::Primitive   primitive;

    if( isRegistered() == 0 )
	primitive = RTController::LayerResize;
    else if( ( getFlags() & RegisteredAsSAP ) != 0 )
	primitive = RTController::LayerResizeSAP;
    else
	primitive = RTController::LayerResizeSPP;

    controller->trace( primitive, this, &new_size );

    if( ( _info->properties & RTPortDescriptor::MaskKind )
			   != RTPortDescriptor::KindUnwired )
    {
	controller->setError( RTController::badOperation,
			      primitive, this, &new_size );
	return 0;
    }

    if( new_size <= 0 )
    {
	controller->setError( RTController::badValue,
			      primitive, this, &new_size );
	return 0;
    }

    if( new_size != old_size )
    {
#if RTS_COUNT
	if( new_size < old_size )
	    controller->countPortDestroy( old_size - new_size );
	else
	    controller->countPortIncarnate( new_size - old_size );
#endif

	RTPortDescriptor * new_info;
	End              * new_ends;
	int                i;

	if( ( getFlags() & Resized ) != 0 )
	    new_info = RTConstCast( RTPortDescriptor, _info );
	else
	{
	    setFlag( Resized );

	    new_info = new RTPortDescriptor;
	    *new_info = *_info;

	    _info    = new_info;
	}

	RTLayerConnector::lock();

	for( i = old_size; --i >= new_size; )
	    unbindAt( i, controller );

	int keep = old_size;

	if( keep > new_size )
	    keep = new_size;

	new_ends = new End [ (unsigned)new_size ];

	End * old_end =    _ends;
	End * new_end = new_ends;

	for( i = keep; --i >= 0; )
	    *new_end++ = *old_end++;

	//lint -save -esym(794,new_end)
	for( i = keep; i < new_size; ++i, ++new_end )
	{
	    new_end->state       = End::Unbound;
	    new_end->remote      = (RTProtocol *)0;
	    new_end->remoteIndex = 0;
	    new_end->signalMap   = (const int *)0;
	}
	//lint -restore

	delete [] _ends;

	_ends                 = new_ends;
	new_info->replication = new_size;

	if( new_size < old_size )
	{
	    RTProtocol_resize_filter filter( controller, this, new_size );

	    controller->filterMessages( owner, filter );
	}

	if( _layer != (RTLayerData *)0 )
	    _layer->resize( controller, old_size );

	RTLayerConnector::unlock();
    }

    return 1;
}
