/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTBindingEnd.h>
#include <RTController.h>
#include <RTDiag.h>
#include <RTMessage.h>
#include <RTPortDescriptor.h>
#include <RTRootProtocol.h>

void RTProtocol::unbindAt( int index, RTController * initiator )
{
    RTAssert( 0 <= index && index < _info->replication );

    End * localEnd = &_ends[ index ];

    if( localEnd->state == End::Unbound )
	return;

    RTProtocol * remote      = localEnd->remote;
    int          remoteIndex = localEnd->remoteIndex;
    End        * remoteEnd   = &remote->_ends[ remoteIndex ];

    RTAssert( remoteEnd->state       == localEnd->state );
    RTAssert( remoteEnd->remote      == this );
    RTAssert( remoteEnd->remoteIndex == index );

    localEnd ->state = End::Unbound;
    remoteEnd->state = End::Unbound;

#if OTRTSDEBUG != DEBUG_NONE
    // Only the debugger might use these fields before the ends are bound again.
    localEnd ->remote      = (RTProtocol *)0;
    localEnd ->remoteIndex = 0;
    remoteEnd->remote      = (RTProtocol *)0;
    remoteEnd->remoteIndex = 0;
#endif

    if( initiator != (RTController *)0 )
    {
	RTProtocol * port = this;
	bool         done = false;

	for(;;)
	{
	    if( port->bindingNotificationRequested() != 0 )
	    {
		RTMessage * msg = initiator->newMsg();

		if( msg == (RTMessage *)0 )
		    initiator->setError( RTController::noMem,
					 RTController::SystemService,
					 (const void *)0,
					 (const void *)0 );
		else
		{
		    RTActor      * owner      = port->getOwner();
		    RTController * controller = owner->context();

		    msg->setSignal( RTRootProtocol::rti_rtUnbound );
		    msg->setPriority( Background );
		    msg->setDestination( owner, port, index );

		    controller->receive( msg, initiator );
		}
	    }

	    if( done )
		break;

	    port  = remote;
	    index = remoteIndex;
	    done  = true;
	}
    }
}
