/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTProtocolDescriptor.h"
#pragma implementation "RTSignalDescriptor.h"
#endif

#include <RTMemoryUtil.h>
#include <RTObject_class.h>
#include <RTProtocolDescriptor.h>
#include <RTSignalDescriptor.h>

// Test whether all signals that might be sent from tx are compatible
// with those that can be received by rx assuming the signals are sorted
// in ascending order according to their names.

static
RTProtocolDescriptor::Status
sendAll( const RTProtocolDescriptor * tx,
	 const RTProtocolDescriptor * rx )
{
    const RTProtocolDescriptor   * conjugate = tx->conjugate;
    const RTSignalDescriptor     * txSignal  = conjugate->inSignals;
    int                            txCount   = conjugate->numInSignals;
    const RTSignalDescriptor     * rxSignal  = rx->inSignals;
    int                            rxCount   = rx->numInSignals;
    RTProtocolDescriptor::Status   result    = RTProtocolDescriptor::ok;

    for( ; --txCount >= 0; ++txSignal, ++rxSignal )
    {
	for( ; ; ++rxSignal )
	{
	    if( --rxCount < 0 )
		// no more signals in rx
		return RTProtocolDescriptor::badSignal;

	    int cmp = RTMemoryUtil::strcmp( txSignal->name, rxSignal->name );

	    if( cmp > 0 )
		continue; // keep looking for the signal

	    if( cmp < 0 )
		// signals are sorted, the one we want should appear
		// before this one: it must not be there
		return RTProtocolDescriptor::badSignal;

	    // found the signal, check the type
	    if( rxSignal->type == (const RTObject_class *)0 )
	    {
		// any type is acceptable
	    }
	    else if( txSignal->type == (const RTObject_class *)0 )
	    {
		// need a specific type
		result = RTProtocolDescriptor::badType;
	    }
	    else if( txSignal->type->isKindOf( rxSignal->type ) == 0 )
	    {
		result = RTProtocolDescriptor::badType;
	    }

	    break; // type is ok
	}
    }

    return result;
}

RTProtocolDescriptor::Status
RTProtocolDescriptor::compare( const RTProtocolDescriptor * other ) const
{
    Status result;

    if( other == conjugate )
    {
	result = ok;
    }
    else if( (result = sendAll( this, other )) == badSignal )
    {
	// no point in checking the other direction
    }
    else switch( sendAll( other, this ) )
    {
	default:
	case badSignal: result = badSignal; break;
	case badType:   result = badType;   break;
	case ok:                            break;
    }

    return result;
}
