/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActorClass.h>
#include <RTActor_class.h>
#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTEventInfo.h>
#include <RTFormat.h>
#include <RTToolsetObserver.h>

static const char * getTypeName( const RTActor_class * info )
{
    if( info == (const RTActor_class *)0 )
	return (const char *)0;

    return info->type->name;
}

int RTToolsetObserver::evtPresence( int                  daemonId,
				    const RTDaemonInfo & daemonInfo,
				    const RTEventInfo  & eventInfo )
{
    const RTDaemon * theDaemon = (RTDaemon *)daemons[ daemonId ];

    if( theDaemon == (const RTDaemon *)0 )
	return 0;

    int shouldHalt = theDaemon->shouldHalt();

    if( theDaemon->shouldReport() == 0 )
	return shouldHalt;

    TO_evtPresence toSendData;
    TO_daemonInfo  toDaemonInfo;

    toDaemonInfo.number   = daemonId;
    toDaemonInfo.invokeID = (char *)0;

    if( eventInfo.observerId     == observerId &&
	structInfo.daemon.number == daemonId ) // it's a request
    {
	toDaemonInfo.invokeID = structInfo.daemon.invokeID;
    }

    toSendData.actor = buildString( daemonInfo.actorPath,
				    eventInfo.actorIndex );

    toSendData.time.sec  = eventInfo.timestamp.tv_sec;
    toSendData.time.nsec = eventInfo.timestamp.tv_nsec;

    char actorID[ 20 ];
    actorID[ 0 ] = '\0';

    if( eventInfo.actor != (const RTActor *)0 )
	(void)RTFormat::_void( (const void *)eventInfo.actor,
			       actorID,
			       (int)sizeof( actorID ) );

    toSendData.actorID = actorID;
    toSendData.classID = getTypeName( eventInfo.info );

    if( TO_encodeEvtPresence( &txBuffer, &toDaemonInfo, &toSendData )
			     == ENC_SUCCESS )
    {
	sendTxBuffer();
    }

    delete [] toSendData.actor;

    return shouldHalt;
}
