/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTMain.h>
#include <RTMemoryUtil.h>
#include <RTNucleus.h>
#include <RTnew.h>
#include <RTstdio.h>

#ifndef TICKS_PER_SECOND
#define TICKS_PER_SECOND 100L
#endif

extern "C"
{

NU_MEMORY_POOL System_Memory;

#ifndef PLUS_MNT

#include <plus/uart.h>

UART_INIT   uart[ 2 ];

extern void __init( void );

extern char __HEAP_END[]; // linker symbol

// This is a modified copy of __init_main from the diab compiler sources.
// It is called from the board-specific Nucleus initialization code.

char ** environ;

void __init_main( void )
{
    extern char ** __env;         // defined in memfile.c
    extern char    __DATA_ROM []; // linker symbols
    extern char    __DATA_RAM [];
    extern char    __DATA_END [];
    extern char    __BSS_START[];
    extern char    __BSS_END  [];

    // copy initialized data from ROM to RAM if at different locations
    if( __DATA_ROM != __DATA_RAM )
	RTMemoryUtil::memcpy( __DATA_RAM, __DATA_ROM, __DATA_END - __DATA_RAM );

    // clear uninitialized data
    if( __BSS_START != __BSS_END )
	RTMemoryUtil::memset( __BSS_START, 0, __BSS_END - __BSS_START );

    // initialization for getenv() and putenv()
    environ = __env;

    // This is where the call to __init() would normally appear.
    // We want to delay constructing global objects until after
    // Nucleus is initialized.

    // This is where the call to main() would occur. We can simply return
    // because main begins immediately after the call to this function.
}

void * malloc( size_t size )
{
    void * memory;

    if( NU_Allocate_Memory( &System_Memory,
			    &memory, size,
			    NU_NO_SUSPEND ) != NU_SUCCESS )
    {
	memory = (void *)0;
    }

    return memory;
}

void free( void * memory )
{
    if( NU_Deallocate_Memory( memory ) != NU_SUCCESS )
	RTDiag::panic( "free" );
}

#endif // PLUS_MNT

void Application_Initialize( void * first_available_memory )
{
    RTNucleus::ticks_per_second = (long)TICKS_PER_SECOND;
    RTNucleus::clock_period     = 1000000000L / (long)TICKS_PER_SECOND;

#ifndef PLUS_MNT
    unsigned MEMORY_SIZE = __HEAP_END - first_available_memory;
#endif

    if( NU_Create_Memory_Pool( &System_Memory,
			       "SysPool",
			       first_available_memory,
			       MEMORY_SIZE,
			       48U, // minimum allocation
			       NU_FIFO ) != NU_SUCCESS )
    {
	RTDiag::panic( "create(general_pool)" );
	return;
    }

#ifndef PLUS_MNT

    __init(); // construct global objects

    void ( * old_lisr )( int );

    if( NU_Register_LISR( SMC1_VECTOR, UART_LISR, &old_lisr ) != NU_SUCCESS )
    {
	RTDiag::panic( "register(uart)" );
	return;
    }

    uart[ SMC1 ].smc_port   = SMC1;
    uart[ SMC1 ].baud_rate  = 9600U;
    uart[ SMC1 ].mode       = SMC_NORMAL_MODE;
    uart[ SMC1 ].read       = 0U;
    uart[ SMC1 ].write      = 0U;
    uart[ SMC1 ].buf_status = BUF_EMPTY;

    if( UART_Init_Port( &uart[ SMC1 ] ) != NU_SUCCESS )
    {
	RTDiag::panic( "init(uart)" );
	return;
    }

#endif // PLUS_MNT

    (void)RTMain::entryPoint( 0, (const char * const *)0 );

    // After we return, Nucleus will switch to our main thread.
}

} // extern "C"
