#! perl -w

# Licensed Materials - Property of IBM                       
# Component IBM Rational ROSE_RT BUILD : TARGET_RTS                 
# Copyright IBM Corporation 1999, 2008. All Rights Reserved.      
# U.S. Government Users Restricted Rights -  Use, duplication or    
# disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.



# Tools to extract the value of a key from the registry.
#
# The registry value corresponding to key\subkey is printed to stdout.
# If the registry value cannot be read for any reason, default will be
# output instead.

# -----------------------------------------------------------------------------
sub tempfile
{
    # Create a tempfile of the form %TMP%\ot<pid>.<NNN>.tmp,
    # where NNN makes the filename unique
    # This is obviously not very efficient...

    my( $prefix, $i, $filename );

    $prefix  = $ENV{'TEMP'} ||  $ENV{'TMP'} || '.';
    $prefix .= '\\' unless $prefix =~ m{\\$};
    $prefix .=  "ot$$";

    for( $i = 0; ; ++$i )
    {
	$filename = "$prefix.$i.tmp";
	return $filename if( ! -e $filename );
    }
}

# -----------------------------------------------------------------------------
sub readfile
{
    my( $filename, $key, $subkey, $result ) = @_;
    my( $foundBlock ) = 0;

    open( REGFILE, "<$filename" ) or return $result;

    # To locate the entry:
    # 1. Scan for [key]
    # 2. Then scan for "subkey"=<value>.
    # 3. Abort scan if cannot find #2 within key block.

    while( <REGFILE> )
    {
	chomp;

	# Some versions of regedit spit out unprintable characters
	# between each character in the resulting file; remove those.
	s{[\x00-\x1B\x80-\xFF]}{}g;

	if( /^\[(.*)]$/ )
	{
	    my( $section ) = $1;

	    last if( $foundBlock );
	    $foundBlock = 1 if $section =~ m/\Q$key\E/i;
	}

	next unless $foundBlock;

	# inside the key block
	if( /^\"(.*)\"=\"(.*)\"$/ )
	{
	    my( $name, $value ) = ( $1, $2 );

	    if( $name =~ m/\Q$subkey\E/i )
	    {
		# Found the key
		$result = $value;
		$result =~ s/\\\\/\\/g;
		last;
	    }
	}
    }

    close( REGFILE );

    return $result;
}

# -----------------------------------------------------------------------------
sub ReadRegEntry
{
    my( $key, $subkey, $result ) = @_;
    my( $tempfile, $status );

    # Create a temporary filename for output of regedit
    $tempfile = &tempfile();

    $status = system( 'regedit', '/e', $tempfile, "\"$key\"" );

    if( $status == 0 )
    {
	# Successfully invoked regedit - now parse file
	$result = readfile( $tempfile, $key, $subkey, $result );
    }

    unlink( $tempfile );

    return $result;
}

1; # for require
